/* von JS ge"andert fuer wildfire, arbeitet auch auf PC */


/* CAUTION: This is the combined ANSI and traditional K&R C version
   of the Numerical Recipes utility file nrutil.c.  Do not confuse
   this file with the same-named file nrutil.c that is supplied in
   the 'recipes' subdirectory.  *That* file contains only ANSI.
   *This* file contains both ANSI and traditional K&R versions,
   along with #ifdef macros to select the correct version.                */

#include <stdlib.h>
#include <stdio.h>

#if (defined(__STDC__) || defined(ANSI) || defined(NRANSI)) /* ANSI */
#define NR_END 0
#define M_ARG size_t
#else /* --- traditional K&R  --- */
#define NR_END 1
#define M_ARG unsigned int  // no size_t ???
#endif /* --- STDC vs. K&R --- */

void nrerror(const char *error_text)
/* Numerical Recipes standard error handler */
{
	fprintf(stderr,"Numerical Recipes run-time error...\n");
	fprintf(stderr,"%s\n",error_text);
	fprintf(stderr,"...now exiting to system...\n");
	exit(1);
}

int *ivector(long nl, long nh)
/* allocate an int vector with subscript range v[nl..nh] */
{
	int *v;

	v=(int *)malloc((M_ARG) ((nh-nl+1+NR_END)*sizeof(int)));
	if (!v) nrerror("allocation failure in ivector()");
	return v-nl+NR_END;
}

double *dvector(long nl, long nh)
/* allocate a double vector with subscript range v[nl..nh] */
{
	double *v;

	v=(double *)malloc((M_ARG) ((nh-nl+1+NR_END)*sizeof(double)));
	if (!v) nrerror("allocation failure in dvector()");
	return v-nl+NR_END;
}


double **dmatrix(long nrl, long nrh, long ncl, long nch)
/* allocate a double matrix with subscript range m[nrl..nrh][ncl..nch] */
{
	long i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
	double **m;

	/* allocate pointers to rows */
	m=(double **) malloc((M_ARG)((nrow+NR_END)*sizeof(double*)));
	if (!m) nrerror("ERROR nrutil.dmatrix().1");
	m += NR_END;
	m -= nrl;

	/* allocate rows and set pointers to them */
	if (((nrow*ncol+NR_END)*sizeof(double))*1.0
	    <(nrow*1.0*ncol+NR_END)*sizeof(double)) {
	  printf("Error dmatrix: darray is to big for your system.\n");
	  exit(1);
	}
	m[nrl]=(double *) malloc((M_ARG)((nrow*ncol+NR_END)*sizeof(double)));
	if (!m[nrl]) nrerror("ERROR nrutil.dmatrix().2");
	m[nrl] += NR_END;
	m[nrl] -= ncl;

	for(i=nrl+1;i<=nrh;i++) m[i]=m[i-1]+ncol;

	/* return pointer to array of pointers to rows */
	return m;
}

int **imatrix(long nrl, long nrh, long ncl, long nch)
/* allocate a int matrix with subscript range m[nrl..nrh][ncl..nch] */
{
	long i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
	int **m;

	/* allocate pointers to rows */
	m=(int **) malloc((M_ARG)((nrow+NR_END)*sizeof(int*)));
	if (!m) nrerror("ERROR nrutil.imatrix().1");
	m += NR_END;
	m -= nrl;


	/* allocate rows and set pointers to them */
	if (((nrow*ncol+NR_END)*sizeof(int))*1.0
	    <(nrow*1.0*ncol+NR_END)*sizeof(int)) {
	  printf("Error imatrix: darray is to big for your system.\n");
	  exit(1);
	}
	m[nrl]=(int *) malloc((M_ARG)((nrow*ncol+NR_END)*sizeof(int)));
	if (!m[nrl]) nrerror("ERROR nrutil.imatrix().2");
	m[nrl] += NR_END;
	m[nrl] -= ncl;

	for(i=nrl+1;i<=nrh;i++) m[i]=m[i-1]+ncol;

	/* return pointer to array of pointers to rows */
	return m;
}

void free_ivector(int *v, long nl, long nh)
/* free an int vector allocated with ivector() */
{
	free((int *) (v+nl-NR_END));  // JS0912 why int* ?? void * !
}

void free_dvector(double *v, long nl, long nh)
/* free a double vector allocated with dvector() */
{
	free((char *) (v+nl-NR_END));
}

void free_dmatrix(double **m, long nrl, long nrh, long ncl, long nch)
/* free a double matrix allocated by dmatrix() */
{
	free((char *) (m[nrl]+ncl-NR_END));
	free((char *) (m+nrl-NR_END));
}

void free_imatrix(int **m, long nrl, long nrh, long ncl, long nch)
/* free an int matrix allocated by imatrix() */
{
	free((char *) (m[nrl]+ncl-NR_END));
	free((char *) (m+nrl-NR_END));
}
